<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="k3b" version="15">
<MenuBar>
    <Menu name="project"><text>&amp;Project</text>
        <Action name="project_add_files" />
        <Action name="project_clear_project" />
    </Menu>

    <Menu name="tools"><text>&amp;Tools</text>
        <Action name="tools_copy_medium" />
        <Action name="tools_format_medium" />
        <Action name="tools_write_image" />
        <Separator />
        <Action name="tools_cdda_rip" />
        <Action name="tools_videocd_rip" />
        <Action name="tools_videodvd_rip" />
    </Menu>

    <Menu name="device"><text>&amp;Device</text>
    </Menu>

    <Menu name="settings">
        <Action name="view_document_header" append="show_merge" />
        <Action name="view_audio_player" append="show_merge" />
    </Menu>

    <Menu name="help"><text>&amp;Help</text>
        <Action name="help_check_system" />    
    </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
    <Action name="file_new" />
    <Action name="file_open" />
    <Action name="file_save" />
    <Separator />
    <Action name="tools_format_medium" />
    <Separator />
    <Action name="location_bar" />
</ToolBar>

<State name="state_project_active">
    <enable>
        <Action name="file_save" />
        <Action name="file_save_as" />
        <Action name="file_close" />
        <Action name="file_close_all" />
        <Action name="project_add_files" />        
        <Action name="project_clear_project" />
    </enable>
</State>
</gui>
